@@ -3,6 +3,6 @@ use strict;
 use warnings;
 
 package WWW::Shopify::Liquid::Filter::Downcase; use base 'WWW::Shopify::Liquid::Filter';
-sub operate { return lc($_[2]); }
+sub operate { return lc(defined $_[2] ? $_[2] : ''); }
 
 1;
\ No newline at end of file
