@@ -34,7 +34,7 @@ use Clone qw(clone);
 
 sub render {
 	my ($self, $hash, $ast) = @_;
-	return '' if !$ast && !wantarray;
+	return ('', $hash) if !$ast && !wantarray;
 	my $hash_clone = $self->clone_hash && $self->clone_hash == 1 ? clone($hash) : $hash;
 	return ('', $hash_clone) unless $ast;
 	my $default_variables = $self->default_variables;
