@@ -2,6 +2,7 @@
 package WWW::Shopify::Liquid::Tag::Output;
 use base 'WWW::Shopify::Liquid::Tag::Free';
 
+use Scalar::Util qw(blessed);
 sub max_arguments { return 1; }
 sub abstract { return 0; }
 
@@ -13,7 +14,9 @@ sub new {
 
 sub operate {
 	my ($self, $hash, $argument) = @_;
-	return $argument;
+	return "$argument" if !ref($argument) || blessed($argument);
+	return '';
+	
 }
 
 1;
\ No newline at end of file
